// Program to exercise the MD_MAX72XX library
//
// Uses most of the functions in the library
#include <MD_MAX72xx.h>
#include <SPI.h>
#include <MD_Parola.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
// Turn on debug statements to the serial output
#define  DEBUG  1

#if  DEBUG
#define PRINT(s, x) { Serial.print(F(s)); Serial.print(x); }
#define PRINTS(x) Serial.print(F(x))
#define PRINTD(x) Serial.println(x, DEC)

#else
#define PRINT(s, x)
#define PRINTS(x)
#define PRINTD(x)


#endif

// Define the number of devices we have in the chain and the hardware interface
// NOTE: These pin numbers will probably not work with your hardware and may
// need to be adapted
#define HARDWARE_TYPE MD_MAX72XX::FC16_HW       
#define MAX_DEVICES  5

#define CLK_PIN   13  // or SCK
#define DATA_PIN  11  // or MOSI
#define CS_PIN    10  // or SS

// SPI hardware interface
MD_MAX72XX mx = MD_MAX72XX(HARDWARE_TYPE, CS_PIN, MAX_DEVICES);
LiquidCrystal_I2C lcd(0x27,2,1,0,4,5,6,7,3,POSITIVE);
// We always wait a bit between updates of the display
#define  DELAYTIME  5  // in milliseconds
int fsrPin = 0;     // the FSR and 10K pulldown are connected to a0
int fsrReading;     // the analog reading from the FSR resistor divider
/* ROTARY ENCODER AND PUSH BUTTON POLLING CODE */
#define Clk 2
#define DT  3
#define SW  4
// Poll and debounces a Keyes rotary and push button switch
// Inputs:
//   Clk (Pin A with 10k pullup resistor)
//   DT  (Pin B with 10k pullup resistor)
//   SW  (Switch without a pullup resistor)
//   +   (Power for pullup resistors)
//   Gnd
// Exports:
//   UpdateSwitch (bool)   [true|false]
//   Switch (byte)         [LOW|HIGH]
//   EncoderPos (char)     [-128..127]
volatile bool UpdateSwitch=false;
volatile byte Switch=HIGH;
volatile char EncoderPos=0;
ISR(TIMER0_COMPB_vect) {
  static byte testClk=(PIND>>Clk)&1;
  static byte testDT=(PIND>>DT)&1;
  static byte lastClk=LOW;
  static byte lastDT=LOW;
  static bool flagClk=false;
  static bool flagDT=false;
  static char encoderDir=0;
  static byte testSW=HIGH;
  static byte statusSW=HIGH;
  static byte cntSW=0;  
  
  // Update Encoder Position
  lastClk=testClk;            // Save Clk
  lastDT=testDT;              // Save DT
  testClk=(PIND>>Clk)&1;      // Get Clk
  testDT=(PIND>>DT)&1;        // Get DT
  if (testClk!=lastClk) {     // Change in Clk?
    flagClk=true;             // Flag Clk has changed
    encoderDir=-1;            // Assume it is the last flag to change
  }
  if (testDT!=lastDT) {       // Change in DT?
    flagDT=true;              // Flag DT has changed
    encoderDir=1;             // Assume it is the last flag to change
  }
  if (flagClk&&flagDT) {      // Both flags have changed
    EncoderPos+=encoderDir;   // Update the encoder
    flagClk=false;            // Reset Clk flag
    flagDT=false;             // Reset DT flag
  }
  
  // Update switch with 10ms debounce
  testSW=(PIND>>SW)&1;
  if (testSW!=statusSW) {
    statusSW=testSW;
    cntSW=10;
  }
  if (cntSW>0) {
    cntSW--;
    if (cntSW==0) {
      Switch=statusSW;
      UpdateSwitch=true;
    }
  }
}


/* MENU SET UP */
char* menuName[]={"Hard Difficulty"};
int menuSize=sizeof(menuName)/sizeof(char*);
char menuValue[]=   {  0};
char menuNumeric[]= {'N'};
char menuValueMin[]={  0};
char menuValueMax[]={  1};
char menuLevel=0;
#define DIFFICULTY 20
void scrollText(char *p)
{
  uint8_t charWidth;
  uint8_t cBuf[8];  // this should be ok for all built-in fonts

  PRINTS("\nScrolling text");
  mx.clear();

  
}

void columns(int fsrReading)
// Demonstrates the use of setColumn()
{
  PRINTS("\nCols 0->max");
  mx.clear();

  for (uint8_t col=0; col<(fsrReading/DIFFICULTY); col++)
  {
    mx.setColumn(col, 0x00);
    delay(DELAYTIME*2);
    mx.setColumn(col, 0xff);
  }
}
bool processMenu(void) {
  static char lastPos=0;
  static char lastMenuLevel=-1;

  // Pre-empt menu level display 
  if (menuLevel!=lastMenuLevel) {
    lastMenuLevel=menuLevel;
    lcd.clear();
    lcd.setCursor(0,0);
    if (menuLevel==0) {
      lcd.print("Menu:");
      lcd.setCursor(0,1);
      lcd.print(menuName[EncoderPos]);
      if (menuNumeric[EncoderPos]=='Y') {
        lcd.print((int)menuValue[EncoderPos]);
      } else {       
        if (menuValue[EncoderPos]!=0) {
          lcd.print("Y");
        } else {
          lcd.print("N");      
        }
      }          
    } else if (menuLevel==1) {
      lcd.print("Set:");
      lcd.setCursor(0,1);
      lcd.print(menuName[lastPos]);
      if (menuNumeric[lastPos]=='Y') {
        lcd.print((int)menuValue[lastPos]);
      } else {       
        if (menuValue[lastPos]!=0) {
          lcd.print("Y");
        } else {
          lcd.print("N");      
        }
      }      
    } else {
      lcd.print("    Light Striker");      
    }
  }
  
  // If push button pushed toggle menuLevel
  if (UpdateSwitch) {
    UpdateSwitch=false;
    if (Switch==LOW) {
      // Re-enter menu if button pushed (for long enough)
      if (menuLevel==-1) {
        menuLevel=0;    // Item menu
        lastPos=1;      // Trigger menu update 
        EncoderPos=0;   // Done
        menuValue[0]=0; // N
      } else {
        // Toggle menu level
        menuLevel=1-menuLevel;
        if (menuLevel==0) {
          // Save item menu position
          EncoderPos=lastPos;
          
          // Exit menu if done!
          if ((EncoderPos==0)&&(menuValue[EncoderPos]!=0)) menuLevel=-1;
        } else {
          // Set value for edit menu
          EncoderPos=menuValue[lastPos];
          
        }
      }
    }
  }
  
  // If encoder turned
  if (menuLevel==0) { // Select menu item
    if (lastPos!=EncoderPos) {
      if (EncoderPos>=menuSize) EncoderPos=0;
      if (EncoderPos<0) EncoderPos=menuSize-1;
      lastPos=EncoderPos;
      
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("Menu:");
      lcd.setCursor(0,1);
      lcd.print(menuName[EncoderPos]);
      if (menuNumeric[EncoderPos]=='Y') {
        lcd.print((int)menuValue[EncoderPos]);
      } else {       
        if (menuValue[EncoderPos]!=0) {
          lcd.print("Y");
        } else {
          lcd.print("N");      
        }
      }

    }
  } else if (menuLevel==1) { // Edit menu item value
    if (menuValue[lastPos]!=EncoderPos) {
      if (EncoderPos>menuValueMax[lastPos]) EncoderPos=menuValueMin[lastPos];
      if (EncoderPos<menuValueMin[lastPos]) EncoderPos=menuValueMax[lastPos];
      menuValue[lastPos]=EncoderPos;
      
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("Set:");
      lcd.setCursor(0,1);
      lcd.print(menuName[lastPos]);
      if (menuNumeric[lastPos]=='Y') {
        lcd.print((int)menuValue[lastPos]);
      } else {       
        if (menuValue[lastPos]!=0) {
          lcd.print("Y");
        } else {
          lcd.print("N");      
        }
      }
      
    }
  }

  return (menuLevel!=-1); // Return true if menu active
}

void setup() {
  mx.begin();
  OCR0B=0xA0;
  TIMSK0|=1<<OCIE0B;

  // Initialise LCD
  pinMode(A4,OUTPUT);
  analogWrite(A4,255);
  lcd.begin(16,2);
  lcd.print("  Light Striker");      
  delay(3000); 
#if  DEBUG
  Serial.begin(9600);
  
#endif
  PRINTS("\n[MD_MAX72XX Test & Demo]");
//  scrollText("MD_MAX72xx Test  ");
}

void loop() {
   if (!processMenu()) { // Menu not active
    // Present the results
    int i;
    int difficulty;
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("Running");
    lcd.setCursor(0,1);
    
      if (menuNumeric[i]=='Y') {
        difficulty = 1;
      } else {       
        difficulty = 0;
      }
      lcd.print(" "); 
   }
   lcd.begin(16,2);
   lcd.clear();
   lcd.print("3...");
   delay(1000);
   lcd.print("2...");
   delay(1000);
   lcd.print("1...");
   delay(1000);
   lcd.print("GO");
   delay(100);
   fsrReading = analogRead(fsrPin); 
  Serial.print("Analog reading = ");
  Serial.println(fsrReading);     // the raw analog reading
  if (fsrReading < 30){
    delay(100);
    lcd.clear();
    lcd.print("Try Again");
    delay(1000);
  }
  else{
    // We'll have a few threshholds, qualitatively determined
    if (fsrReading < 10) {
      Serial.println(" - No pressure");
    } else if (fsrReading < 200) {
      Serial.println(" - Light touch");
    } else if (fsrReading < 500) {
      Serial.println(" - Light squeeze");
    } else if (fsrReading < 800) {
      Serial.println(" - Medium squeeze");
    } else {
      Serial.println(" - Big squeeze");
    }
    // put your main code here, to run repeatedly:
    columns(fsrReading);
    lcd.clear();
    lcd.print("Good Job");
    delay(3000);
    mx.clear();
  }
   
}
